@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -21,10 +21,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -257,15 +253,32 @@ void
 print_jobs (net_id *fd)
 {
 	child_t *ret_child;
+	char *s, is_first = 1;
+
+	s = "{ \"success\" : true, \"data\" : [ ";
+	net_write (fd, s, schedwi_strlen (s));
 
 	lwc_rewindLL (list_jobs);
 	while ((ret_child = lwc_nextLL (list_jobs)) != NULL) {
 		if (ret_child->job_id != NULL) {
+			if (is_first) {
+				is_first = 0;
+				s = "\"";
+			}
+			else {
+				s = ", \"";
+			}
+			net_write (fd, s, schedwi_strlen (s));
+				
 			net_write (	fd, ret_child->job_id,
 					schedwi_strlen (ret_child->job_id));
-			net_write (fd, " ", 1);
+			s = "\"";
+			net_write (fd, s, schedwi_strlen (s));
 		}
 	}
+
+	s = " ] }";
+	net_write (fd, s, schedwi_strlen (s));
 }
 
 /*-----------------============== End Of File ==============-----------------*/
