@@ -260,6 +260,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 #if HAVE_SETRLIMIT
 	struct rlimit rlim;
 #endif
+#if HAVE_GETPRIORITY && HAVE_SETPRIORITY
+	int old_nice;
+#endif
 
 #if HAVE_ASSERT_H
 	assert (j != NULL);
@@ -319,8 +322,13 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 		/* No error, the job is running */
 		if (nb_read == -1 || nb_read == 0) {
 			close (status_pipe_father[1]);
-			/* Run the main function of the launcher */
-			launcher_main (child, job_id);
+			if (j->detach == 0) {
+				/*
+				 * Run the main function of the launcher to
+				 * wait for the ccommand result
+				 */
+				launcher_main (child, job_id);
+			}
 			exit (0);
 		}
 
@@ -358,8 +366,14 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 
 #if HAVE_GETPRIORITY && HAVE_SETPRIORITY
 	/* nice */
-	if (	   getpriority (PRIO_PROCESS, 0) != j->nice
-		&& setpriority (PRIO_PROCESS, 0, j->nice) != 0)
+	old_nice = getpriority (PRIO_PROCESS, 0);
+	/*
+	 * In GNU/Hurd, getpriority() returns strange values (not in -20/+19).
+	 * In this case, do nothing.
+	 */
+	if (   old_nice >= -20 && old_nice <= 19 && old_nice != j->nice
+	    && j->nice >= -20 && j->nice <= 19
+	    && setpriority (PRIO_PROCESS, 0, j->nice) != 0)
 	{
 		unused = write (3, &errno, sizeof (int));
 		child_err = CHILD_SETPRIORITY;
@@ -739,6 +753,7 @@ submit_job (	job_parameters_t *j, const char *job_id, int *job_pid,
 			build_errmsg (j->command, _("No such file"), err_msg);
 			return -2;
 	}
+	add_detach_job_run (&jr, j->detach);
 
 	/* Run the command */
 	if (exec_command (&jr, job_id, job_pid, err_msg) != 0) {
