@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -24,10 +24,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 /* Allow the use in C++ code.  */
 #ifdef __cplusplus
 extern "C" {
