@@ -32,6 +32,12 @@
  *       load_env --> Do the user environement should be load (1) or
  *                    not (0).  If yes (1), the user shell is run with the
  *                    command as a parameter (ie. /bin/sh -c command)
+ *         detach --> Whether the job should be start in detach mode or not.
+ *                    In detach mode, Schedwi will not wait on the end of the
+ *                    job.  This may be usefull to start a daemon, a service
+ *                    or a long running process.  Only the success of the
+ *                    start of the job is reported (basically if the provided
+ *                    command can be exec)
  *      arguments --> Arguments list
  *        environ --> Environement to load (several other variables will be
  *                    added, like SHELL, PATH, LOGNAME, ...)
@@ -65,6 +71,7 @@ init_job_parameters (job_parameters_t *j)
 		j->file_stderr = NULL;
 		j->cgroup_name = NULL;
 		j->load_env = 0;
+		j->detach = 0;
 		init_argument (&(j->arguments));
 		init_environment (&(j->environ));
 	}
@@ -245,6 +252,18 @@ add_loadenv_job_parameters (job_parameters_t *j, int flag)
 }
 
 
+/*
+ * Set the `detach mode' flag
+ */
+void
+add_detach_job_parameters (job_parameters_t *j, int flag)
+{
+	if (j != NULL) {
+		j->detach = flag;
+	}
+}
+
+
 /*
  * Add an argument
  *
