@@ -56,6 +56,7 @@ struct job_parameters {
 	char *file_stderr;
 	char *cgroup_name;
 	int load_env;
+	int detach;
 	argument_t arguments;
 	environment_t environ;
 };
@@ -80,6 +81,7 @@ extern int add_cgroup_job_parameters OF((	job_parameters_t *j,
 						const char *cgroup_name,
 						size_t len));
 extern void add_loadenv_job_parameters OF((job_parameters_t *j, int flag));
+extern void add_detach_job_parameters OF((job_parameters_t *j, int flag));
 extern int add_arg_job_parameters OF((	job_parameters_t *j, const char *arg,
 					size_t len));
 extern int add_env_job_parameters OF((	job_parameters_t *j,
