@@ -80,6 +80,7 @@ struct job_run {
 	char *cgroup_name;
 	char *work;
 	char *shell;
+	int detach;
 
 #if HAVE_UID_T
 	uid_t uid;
@@ -108,6 +109,7 @@ extern int add_username_job_run OF((job_run_t *j, const char *username));
 extern int add_outfile_job_run OF((	job_run_t *j, const char *file_stdout,
 					const char *file_stderr));
 extern int add_cgroup_job_run OF((job_run_t *j, const char *cgroup_name));
+extern int add_detach_job_run OF((job_run_t *j, int detach));
 extern int add_command_job_run OF((	job_run_t *j, const char *command,
 					const argument_t *args, int load_env,
 					int *idx));
