@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -24,23 +24,19 @@
  *    // Using the syslog mechanism
  *    int n = 0;
  *    lwc_newLog ("my_program", 1, LOG_LOCAL0, NULL, NULL);
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
  *    lwc_closeLog ();
  *
- *    // Using a log file 
+ *    // Using a log file
  *    lwc_newLog (NULL, 0, 0, "/tmp/my_program.log", "%m/%d/%Y");
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
- *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++); 
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
+ *    lwc_writeLog (LOG_INFO, "sample information message number %d", n++);
  *    lwc_closeLog ();
  */
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <string.h>
 #endif
@@ -322,7 +318,7 @@ lwc_vwriteLog (int level, const char *format, va_list ap)
 	}
 	else {
 #if HAVE_SYSLOG
-		syslog (level, buffer);
+		syslog (level, buffer, 0);
 #endif
 	}
 /*PTHREAD	pthread_mutex_unlock(&lock); */
