@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -51,9 +51,6 @@
 #if HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
 #endif
 
 #if HAVE_UNISTD_H
@@ -75,6 +72,14 @@
 extern int errno;
 #endif
 
+#if HAVE_TCPD_H
+#include <tcpd.h>
+#endif
+
+#if HAVE_SYSLOG_H
+#include <syslog.h>
+#endif
+
 #if HAVE_ASSERT_H
 #include <assert.h>
 #endif
@@ -95,19 +100,39 @@ extern int errno;
 #define closesocket(x) close(x)
 #endif
 
+#ifndef IPV6_V6ONLY
+#define IPV6_V6ONLY 26
+#endif
+
+#ifndef IPPROTO_IPV6
+#define IPPROTO_IPV6 41
+#endif
+
+#ifndef NI_MAXHOST
+#define NI_MAXHOST 1025
+#endif
+
 #include <lib_functions.h>
 #include <lwc_log.h>
-#include <lib_functions.h>
+#include <utils.h>
 #include <net_utils_sock.h>
+#include <addrmatch.h>
+#include <JSON_parser.h>
+#include <parse_json_result.h>
 
 
 #define BUFFER_LEN 1024
-#define ENF_OF_REQUEST "\nbYe"
 
+#if HAVE_LIBWRAP
+int allow_severity;
+int deny_severity;
+#endif
+
+
+#ifndef HAVE_GETADDRINFO
 /* Protocol number for TCP */
 static int protocol = -1;
 
-
 /*
  *  Convert a port number/name in an unsigned short int suitable for the
  *  member sin_port of the sockaddr_in structure
@@ -160,19 +185,36 @@ get_port_number (const char *port)
 
 
 /*
- * Check that the network client is allowed
+ * Convert the provided port string (a service name or number) to a port
+ * number
  *
  * Return:
- *   0 --> Client allowed
- *   1 --> Client not allowed
- *  -1 --> Error (a message is displayed using lwc_writeLog())
+ *   The new string (to be freed by the caller by free() or
+ *   NULL in case of memory allocation error
  */
-static int
-net_check_client (struct sockaddr_in *in, const char *schedwi_server_name)
+char *
+port2number (const char *port)
 {
-	char *addr;
-	int i, try_no;
-	struct hostent *he;
+	char *s;
+
+#if HAVE_ASSERT_H
+	assert (port != NULL);
+#endif
+	s = (char *) malloc (7);
+	if (s == NULL) {
+		return NULL;
+	}
+	copy_ulltostr (ntohs (get_port_number (port)), s);
+	return s;
+}
+#else /* HAVE_GETADDRINFO */
+char *
+port2number (const char *port)
+{
+	char *s;
+	int try_no, ret;
+	struct addrinfo hints;
+	struct addrinfo *result;
 #if HAVE_NANOSLEEP
 	struct timespec req;
 
@@ -180,62 +222,221 @@ net_check_client (struct sockaddr_in *in, const char *schedwi_server_name)
 	req.tv_nsec = 0;
 #endif
 
-	if (in == NULL) {
-		return 1;
+
+#if HAVE_ASSERT_H
+	assert (port != NULL);
+#endif
+	s = (char *) malloc (7);
+	if (s == NULL) {
+		return NULL;
 	}
 
-	addr = inet_ntoa (in->sin_addr);
-	if (schedwi_strcasecmp (addr, schedwi_server_name) == 0) {
-		return 0;
-	}
+	schedwi_memset (&hints, 0, sizeof (struct addrinfo));
+	hints.ai_family    = AF_UNSPEC;
+	hints.ai_socktype  = SOCK_STREAM; /* Stream socket */
+	hints.ai_flags     = 0;
+        hints.ai_protocol  = 0;           /* Any protocol */
+	hints.ai_canonname = NULL;
+	hints.ai_addr      = NULL;
+	hints.ai_next      = NULL;
 
 	try_no = 0;
 	do {
-		he = gethostbyaddr (	&(in->sin_addr),
-					sizeof (in->sin_addr),
-					AF_INET);
-		if (he == NULL) {
-			if (h_errno == TRY_AGAIN) {
-				try_no++;
-				if (try_no < SCHEDWI_HOSTLOOKUP_RETRIES) {
+		ret = getaddrinfo (NULL, port, &hints, &result);
+		/*
+		 * If the name server returned a temporary failure, try
+		 * again.
+		 */
+		if (ret == EAI_AGAIN) {
+			try_no++;
+			if (try_no < SCHEDWI_HOSTLOOKUP_RETRIES) {
 #if HAVE_NANOSLEEP
-					nanosleep (&req, NULL);
+				nanosleep (&req, NULL);
 #elif HAVE_SLEEP
-					sleep (SCHEDWI_HOSTLOOKUP_SLEEP);
+				sleep (SCHEDWI_HOSTLOOKUP_SLEEP);
 #elif HAVE_USLEEP
-					usleep (  SCHEDWI_HOSTLOOKUP_SLEEP
-						* 1000000);
+				usleep (SCHEDWI_HOSTLOOKUP_SLEEP * 1000000);
 #endif
+			}
+		}
+		else {
+			if (ret != 0) {
+				strncpy (s, port, 6);
+				s[6] = '\0';
+				return s;
+			}
+		}
+	} while (ret != 0 && try_no < SCHEDWI_HOSTLOOKUP_RETRIES);
+
+	if (ret != 0) {
+		strncpy (s, port, 6);
+		s[6] = '\0';
+		return s;
+	}
+
+	ret = getnameinfo (	result->ai_addr, result->ai_addrlen,
+				NULL, 0,
+				s, 7,
+				NI_NUMERICSERV);
+	freeaddrinfo (result);
+	if (ret != 0) {
+		strncpy (s, port, 6);
+		s[6] = '\0';
+		return s;
+	}
+	return s;
+}
+#endif /* HAVE_GETADDRINFO */
+
+
+/*
+ * Retrieve the next waiting connection from the provided set.
+ * sock_set and nb_set are updated accordingly.
+ *   sock_set --> Set of waiting sockets
+ *   max_set  --> Highest socket number + 1 in the set
+ *   nb_set   --> Number of waiting sockets in the set
+ *   allow_from --> Only connection from the server are allowed
+ *
+ * Return :
+ *   The next socket
+ *   -1 in case of error (errno is set and a message is displayed using
+ *      lwc_writeLog())
+ *   -2 No more socket waiting (the set is empty)
+ */
+static int
+net_accept_next (fd_set *sock_set, int max_set, int *nb_set,
+		 const char *allow_from)
+{
+	int i, nb, save_errno, s, ret;
+#if HAVE_GETADDRINFO
+	struct sockaddr_storage sockname;
+#else
+	struct sockaddr_in sockname;
+#endif
+#if HAVE_GETNAMEINFO
+	char name[NI_MAXHOST];
+#endif
+	socklen_t len;
+#if HAVE_LIBWRAP
+	struct request_info req;
+#endif
+
+	nb = *nb_set;
+	for (i = 0 ; i < max_set && nb > 0; i++) {
+		if (FD_ISSET (i, sock_set) == 0) {
+			continue;
+		}
+		nb--;
+
+		do {
+			len = sizeof (sockname);
+			s = accept (i, (struct sockaddr *)&sockname, &len);
+			if (s < 0) {
+#if CYGWIN_IN_USE
+				/*
+				 * Sometimes, using Cygwin, the errno value is
+				 * not set to EINTR nor EAGAIN when accept has
+				 * been interrupted by a signal.  When accept()
+				 * returns -1, the error is then ignored and
+				 * accept() is run again
+				 */
+				continue;
+#else
+				if (errno != EINTR && errno != EAGAIN) {
+					save_errno = errno;
+					lwc_writeLog (	LOG_CRIT,
+						_("Network error: accept: %s"),
+						strerror (errno));
+					errno = save_errno;
+					return -1;
 				}
+#endif
 			}
-			else {
-			if (h_errno == NO_RECOVERY) {
+		} while (s < 0);
+
+		FD_CLR (i, sock_set);
+		(*nb_set)--;
+
+		/*
+		 * Only the server (defined by the configuration option
+		 * ALLOW_FROM) is allowed.
+		 * It's always refused if the variable is not set (it should
+		 * not happened as if the variable is not set in the
+		 * configuration file it is set in schedwiclnt.c to the value
+		 * of the SERVER variable)
+		 */
+		if (allow_from == NULL || allow_from[0] == '\0') {
+			net_close_sock (s);
+			continue;
+		}
+
+#if HAVE_GETNAMEINFO
+		ret = getnameinfo (	(struct sockaddr *)(&sockname), len,
+					name, sizeof (name), NULL, 0,
+					NI_NUMERICHOST);
+		if (ret != 0) {
+			lwc_writeLog (	LOG_NOTICE,
+					_("getnameinfo: %s"),
+					gai_strerror (ret));
+			net_close_sock (s);
+			continue;
+		}
+#else /* no HAVE_GETNAMEINFO */
+		ret = sizeof (name) - 1;
+		strncpy (name, inet_ntoa (sockname.sin_addr), ret);
+		name[ret] = '\0';
+#endif /* HAVE_GETNAMEINFO */
+
+#if HAVE_LIBWRAP
+		/* Check the network access using the TCP Wrappers */
+		allow_severity = LOG_INFO;
+		deny_severity  = LOG_WARNING;
+
+		request_init (	&req,
+				RQ_DAEMON, SCHEDWI_TCP_WRAPPERS_DAEMON,
+				RQ_FILE, s, 0);
+		fromhost (&req);
+		if (hosts_access (&req) == 0) {
+			lwc_writeLog (	LOG_DEBUG,
+			_("Connection refused by the TCP Wrappers for %s"),
+					name);
+			net_close_sock (s);
+			continue;
+		}
+#endif /* HAVE_LIBWRAP */
+
+		/*
+		 * Check through the internal network access list if the client
+		 * (the Schedwi server) is allowed.  This list is set through
+		 * the configuration variable ALLOW_FROM.
+		 */
+		ret = addr_match_list (name, allow_from);
+		switch (ret) {
+			case 1:
+				/* Allowed */
+				return s;
+			case 0:
+				lwc_writeLog (	LOG_DEBUG,
+						_("Connection refused for %s"),
+						name);
+				break;
+			case -1:
+				lwc_writeLog (	LOG_CRIT,
+						_("Memory allocation error"));
+				break;
+			case -2:
+				/*
+				 * Should not occur (checked at start time
+				 * by check_netlist())
+				 */
 				lwc_writeLog (	LOG_ERR,
-						_("Name server error"));
-				return -1;
-			}
-			else {
-				return 1;
-			}}
+	_("Inconsistent mask length in one of the address in ALLOW_FROM"));
+				break;
 		}
-	} while (he == NULL && try_no < SCHEDWI_HOSTLOOKUP_RETRIES);
-	if (he == NULL) {
-		lwc_writeLog (LOG_WARNING, _("Name server unavailable"));
-		return -1;
-	}
 
-	if (schedwi_strcasecmp (he->h_name, schedwi_server_name) == 0) {
-		return 0;
+		net_close_sock (s);
 	}
-
-	for (i = 0; (he->h_aliases)[i] != NULL; i++) {
-		if (schedwi_strcasecmp ((he->h_aliases)[i],
-					schedwi_server_name) == 0)
-		{
-			return 0;
-		}
-	}
-	return 1;
+	return -2;
 }
 
 
@@ -250,68 +451,125 @@ net_check_client (struct sockaddr_in *in, const char *schedwi_server_name)
  *      lwc_writeLog())
  */
 int
-net_accept_sock (int sock, const char *schedwi_server_name)
+net_accept_sock (fd_set *sock_set, int sock_max, const char *allow_from)
 {
-	struct sockaddr_in sockname;
-#if HAVE_SOCKLEN_T
-	socklen_t len;
-#else
-	int len;
-#endif
+	static fd_set waiting_set;
+	static int still_waiting = 0;
 	int s, ret, save_errno;
+	fd_set rfds;
+
+#if HAVE_ASSERT_H
+	assert (sock_set != NULL);
+#endif
+
+	/* First, manage the connections still waiting from a previous call */
+	s = net_accept_next (	&waiting_set, sock_max, &still_waiting,
+				allow_from);
+	if (s == -1) {
+		return -1;
+	}
+	if (s >= 0) {
+		return s;
+	}
+
+	/* No more pending connections from a previous call */
 
 	while (1) {
-		len = sizeof(struct sockaddr_in);
-		s = accept (sock, (struct sockaddr *)&sockname, &len);
-		if (s < 0) {
+		rfds = *sock_set;
+		ret = select (sock_max, &rfds, NULL, NULL, NULL);
+		if (ret < 0) {
 #if CYGWIN_IN_USE
 			/*
 			 * Sometimes, using Cygwin, the errno value is
-			 * not set to EINTR nor EAGAIN when accept has
-			 * been interrupted by a signal.  When accept()
-			 * returns -1, the error is then ignored and accept()
+			 * not set to EINTR nor EAGAIN when select has
+			 * been interrupted by a signal.  When select()
+			 * returns -1, the error is then ignored and select()
 			 * is run again
 			 */
 			continue;
 #else
-			if (	   errno == EINTR
-				|| errno == EAGAIN)
-			{
+			if (errno == EINTR || errno == EAGAIN) {
 				continue;
 			}
 			save_errno = errno;
 			lwc_writeLog (	LOG_CRIT,
-					_("Network error: accept: %s"),
+					_("Network error: select: %s"),
 					strerror (errno));
 			errno = save_errno;
 			return -1;
 #endif
 		}
+		waiting_set = rfds;
+		still_waiting = ret;
 
-		/*
-		 * Only the server (defined by the configuration option
-		 * SERVER) is allowed
-		 */
-		if (	   schedwi_server_name != NULL
-			&& schedwi_server_name[0] != '\0')
-		{
-			ret = net_check_client (&sockname, schedwi_server_name);
-			if (ret == 1) {
-				lwc_writeLog (	LOG_NOTICE,
-						_("Connection refused for %s"),
-						inet_ntoa(sockname.sin_addr));
-			}
-			if (ret != 0) {
-				closesocket (s);
-				continue;
-			}
+		s = net_accept_next (	&waiting_set, sock_max, &still_waiting,
+					allow_from);
+		if (s == -1) {
+			return -1;
+		}
+		if (s >= 0) {
+			return s;
 		}
-		return s;
 	}
 	return -1;
 }
 
 
+/*
+ * Create a socket (plus bind and listen)
+ * domain, type, protocol_s : see socket(2)
+ * addr, addrlen  : see bind(2)
+ *
+ * Return:
+ *   The socket OR
+ *   -1 in case of error
+ */
+static int
+net_create_sock (int domain, int type, int protocol_s,
+		const struct sockaddr *addr, socklen_t addrlen)
+{
+	int sock, opt;
+
+	/* Create the socket */
+	sock = socket (domain, type, protocol_s);
+	if (sock < 0) {
+		lwc_writeLog (LOG_ERR, _("socket: %s"), strerror (errno));
+		return -1;
+	}
+
+	/* Set the REUSEADDR flag */
+	opt = 1;
+	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof (opt)) != 0)
+	{
+		lwc_writeLog (LOG_ERR, _("setsockopt: %s"), strerror (errno));
+	}
+
+	/* Allow the IPv6 bind to the same port as in IPv4 */
+	if (	   domain == AF_INET6
+		&& setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY,
+				&opt, sizeof (opt)) != 0)
+	{
+		lwc_writeLog (LOG_ERR, _("setsockopt: %s"), strerror (errno));
+	}
+
+	/* Bind to the address */
+	if (bind (sock, addr, addrlen) != 0) {
+		lwc_writeLog (LOG_ERR, _("bind: %s"), strerror (errno));
+		closesocket (sock);
+		return -1;
+	}
+
+	/* Prepare to accept requests */
+	if (listen (sock, SCHEDWI_LISTEN_BACKLOG) != 0) {
+		lwc_writeLog (LOG_ERR, _("listen: %s"), strerror (errno));
+		closesocket (sock);
+		return -1;
+	}
+
+	return sock;
+}
+
+
 /*
  * Create a socket ready to accept requests
  *
@@ -321,18 +579,30 @@ net_accept_sock (int sock, const char *schedwi_server_name)
  *             the interfaces)
  *
  * Return:
- *   The socket or
- *   -1 in case of error (an error message is displayed using lwc_writeLog())
+ *    0 --> No error (sock_set and sock_max are set)
+ *   -1 --> Error (an error message is displayed using lwc_writeLog())
  */
 int
-net_server_sock (const char *port_number, const char *iface)
+net_server_sock (const char *port_number, const char *iface,
+		const char *address_family,
+		fd_set *sock_set, int *sock_max)
 {
+	int sock, try_no;
+	fd_set fds;
+#if HAVE_GETADDRINFO
+	struct addrinfo hints;
+	struct addrinfo *result, *rp;
+	int ret;
+	int max_sock;
+#else /* not HAVE_GETADDRINFO */
+#if HAVE_GETPROTOBYNAME
 	struct protoent *prot;
-	int sock, opt, try_no;
+#endif
 	struct  sockaddr_in sad;
 	unsigned long int iface_addr;
 	unsigned short int port;
 	struct hostent *he;
+#endif /* HAVE_GETADDRINFO */
 #if HAVE_NANOSLEEP
 	struct timespec req;
 
@@ -344,6 +614,100 @@ net_server_sock (const char *port_number, const char *iface)
 	assert (port_number != NULL);
 #endif
 
+/*
+ * If the getaddrinfo() function is available lets use it (support for IPv6)
+ */
+#if HAVE_GETADDRINFO
+
+	schedwi_memset (&hints, 0, sizeof (struct addrinfo));
+	hints.ai_family = AF_UNSPEC;   /* Allow IPv4 and IPv6 */
+	if (address_family != NULL) {
+		if (schedwi_strcasecmp (address_family, "inet") == 0) {
+			hints.ai_family = AF_INET; /* Only allow IPv4 */
+		}
+		else {
+			if (schedwi_strcasecmp (address_family, "inet6") == 0) {
+				hints.ai_family = AF_INET6; /* Only IPv6 */
+			}
+		}
+	}
+	hints.ai_socktype  = SOCK_STREAM; /* Stream socket */
+	hints.ai_flags     = AI_PASSIVE;
+        hints.ai_protocol  = 0;           /* Any protocol */
+	hints.ai_canonname = NULL;
+	hints.ai_addr      = NULL;
+	hints.ai_next      = NULL;
+
+	try_no = 0;
+	do {
+		ret = getaddrinfo (
+			(iface == NULL || iface[0] == '\0') ? NULL : iface,
+			port_number, &hints, &result);
+		/*
+		 * If the name server returned a temporary failure, try
+		 * again.
+		 */
+		if (ret == EAI_AGAIN) {
+			try_no++;
+			if (try_no < SCHEDWI_HOSTLOOKUP_RETRIES) {
+#if HAVE_NANOSLEEP
+				nanosleep (&req, NULL);
+#elif HAVE_SLEEP
+				sleep (SCHEDWI_HOSTLOOKUP_SLEEP);
+#elif HAVE_USLEEP
+				usleep (SCHEDWI_HOSTLOOKUP_SLEEP * 1000000);
+#endif
+			}
+		}
+		else {
+			if (ret != 0) {
+				lwc_writeLog (LOG_ERR, _("getaddrinfo: %s"),
+						gai_strerror (ret));
+				return -1;
+			}
+		}
+	} while (ret != 0 && try_no < SCHEDWI_HOSTLOOKUP_RETRIES);
+
+	if (ret != 0) {
+		lwc_writeLog (LOG_WARNING, _("Name server unavailable"));
+		return -1;
+	}
+
+	/* Create the sockets for all the returned addresses */
+	FD_ZERO (&fds);
+	max_sock = -1;
+	for (rp = result; rp != NULL; rp = rp->ai_next) {
+		sock = net_create_sock (rp->ai_family,
+					rp->ai_socktype,
+					rp->ai_protocol,
+					rp->ai_addr,
+					rp->ai_addrlen);
+		if (sock == -1) {
+			/* Close all the previously opened sockets */
+			while (max_sock >= 0) {
+				if (FD_ISSET (max_sock, &fds) != 0) {
+					closesocket (max_sock);
+				}
+				max_sock--;
+			}
+			freeaddrinfo (result);
+			return -1;
+		}
+		FD_SET (sock, &fds);
+		if (sock > max_sock) {
+			max_sock = sock;
+		}
+	}
+
+	freeaddrinfo (result);
+
+	*sock_set = fds;
+	*sock_max = max_sock + 1;
+	return 0;
+
+
+#else /* not HAVE_GETADDRINFO */
+
 	/* Get the port number */
 	port = get_port_number (port_number);
 	if (port == 0) {
@@ -417,41 +781,26 @@ net_server_sock (const char *port_number, const char *iface)
 #endif
 	}
 
-	/* Create the socket */
-	sock = socket (PF_INET, SOCK_STREAM, protocol);
-	if (sock < 0) {
-		lwc_writeLog (LOG_ERR, _("socket: %s"), strerror (errno));
-		return -1;
-	}
-
-	/* Set the REUSEADDR flag */
-	opt = -1;
-	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof (opt)) != 0)
-	{
-		lwc_writeLog (LOG_ERR, _("setsockopt: %s"), strerror (errno));
-	}
-
 	/* Compose the address */
-	schedwi_memset(&sad, 0, sizeof(sad));
+	schedwi_memset (&sad, 0, sizeof(sad));
         sad.sin_family = AF_INET;
         sad.sin_addr.s_addr = iface_addr;
 	sad.sin_port = port;
 
-	/* Bind to the address */
-	if (bind (sock, (struct sockaddr *)&sad, sizeof(sad)) != 0) {
-		lwc_writeLog (LOG_ERR, _("bind: %s"), strerror (errno));
-		closesocket (sock);
+	/* Create the socket */
+	sock = net_create_sock (PF_INET, SOCK_STREAM, protocol,
+				(struct sockaddr *)&sad, sizeof(sad));
+
+	if (sock < 0) {
 		return -1;
 	}
 
-	/* Prepare to accept requests */
-	if (listen (sock, SCHEDWI_LISTEN_BACKLOG) != 0) {
-		lwc_writeLog (LOG_ERR, _("listen: %s"), strerror (errno));
-		closesocket (sock);
-		return -1;
-	}
-
-	return sock;
+	FD_ZERO (&fds);
+	FD_SET (sock, &fds);
+	*sock_set = fds;
+	*sock_max = sock + 1;
+	return 0;
+#endif /* HAVE_GETADDRINFO */
 }
 
 
@@ -467,12 +816,21 @@ net_server_sock (const char *port_number, const char *iface)
 int
 net_client_sock (const char *port_number, const char *hostname)
 {
-	struct protoent *prot;
 	int sock, try_no;
+#if HAVE_GETADDRINFO
+	struct addrinfo hints;
+	struct addrinfo *result, *rp;
+	int ret;
+#else /* not HAVE_GETADDRINFO */
+#if HAVE_GETPROTOBYNAME
+	struct protoent *prot;
+#endif
 	struct  sockaddr_in sad;
 	unsigned long int iface_addr;
 	unsigned short int port;
 	struct hostent *he;
+#endif /* HAVE_GETADDRINFO */
+
 #if HAVE_NANOSLEEP
 	struct timespec req;
 
@@ -484,6 +842,78 @@ net_client_sock (const char *port_number, const char *hostname)
 	assert (port_number != NULL && hostname != NULL);
 #endif
 
+/*
+ * If the getaddrinfo() function is available lets use it (support for IPv6)
+ */
+#if HAVE_GETADDRINFO
+
+	schedwi_memset (&hints, 0, sizeof (struct addrinfo));
+	hints.ai_family   = AF_UNSPEC;     /* Allow IPv4 or IPv6 */
+	hints.ai_socktype = SOCK_STREAM;   /* Stream socket */
+#ifdef AI_ADDRCONFIG
+	hints.ai_flags    = AI_ADDRCONFIG;
+#endif
+	hints.ai_protocol = 0;             /* Any protocol */
+
+	try_no = 0;
+	do {
+		ret = getaddrinfo (hostname, port_number, &hints, &result);
+		/*
+		 * If the name server returned a temporary failure, try
+		 * again.
+		 */
+		if (ret == EAI_AGAIN) {
+			try_no++;
+			if (try_no < SCHEDWI_HOSTLOOKUP_RETRIES) {
+#if HAVE_NANOSLEEP
+				nanosleep (&req, NULL);
+#elif HAVE_SLEEP
+				sleep (SCHEDWI_HOSTLOOKUP_SLEEP);
+#elif HAVE_USLEEP
+				usleep (SCHEDWI_HOSTLOOKUP_SLEEP * 1000000);
+#endif
+			}
+		}
+		else {
+			if (ret != 0) {
+				lwc_writeLog (LOG_ERR, _("getaddrinfo: %s"),
+						gai_strerror (ret));
+				return -1;
+			}
+		}
+	} while (ret != 0 && try_no < SCHEDWI_HOSTLOOKUP_RETRIES);
+
+	if (ret != 0) {
+		lwc_writeLog (LOG_WARNING, _("Name server unavailable"));
+		return -1;
+	}
+
+	/* Try the addresses one after the other until one is working */
+	sock = -1;
+	for (rp = result; rp != NULL; rp = rp->ai_next) {
+		sock = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+		if (sock < 0) {
+			continue;
+		}
+
+		if (connect (sock, rp->ai_addr, rp->ai_addrlen) == 0) {
+			break;
+		}
+		closesocket (sock);
+		sock = -1;
+	}
+
+	freeaddrinfo (result);
+	if (sock < 0) {
+		lwc_writeLog (LOG_ERR, _("Failed to connect to %s: %s"),
+				hostname, port_number);
+		return -1;
+	}
+
+	return sock;
+
+#else /* not HAVE_GETADDRINFO */
+		
 	/* Get the port number */
 	port = get_port_number (port_number);
 	if (port == 0) {
@@ -572,6 +1002,34 @@ net_client_sock (const char *port_number, const char *hostname)
 	}
 
 	return sock;
+#endif /* HAVE_GETADDRINFO */
+}
+
+
+/*
+ * JSON callback function
+ */
+static int
+json_char_callback (void* ctx, int type, const JSON_value* value)
+{
+	int *nb_open_brackets = (int *)ctx;
+
+	switch (type) {
+		case JSON_T_ARRAY_BEGIN:
+		case JSON_T_OBJECT_BEGIN:
+			if (*nb_open_brackets < 0) {
+				*nb_open_brackets = 1;
+			}
+			else {
+				(*nb_open_brackets)++;
+			}
+			break;
+		case JSON_T_ARRAY_END:
+		case JSON_T_OBJECT_END:
+			(*nb_open_brackets)--;
+			break;
+	}
+	return 1;
 }
 
 
@@ -594,7 +1052,9 @@ net_read_sock (int sock, char **buff, size_t *len)
 	char *b, *tmp;
 	fd_set rfds;
 	struct timeval tv;
-	int ret;
+	int ret, nb_open_brackets, i;
+	JSON_config config;
+	struct JSON_parser_struct *jc;
 
 #if HAVE_ASSERT_H
 	assert (buff != NULL && len != NULL);
@@ -610,6 +1070,18 @@ net_read_sock (int sock, char **buff, size_t *len)
 		*len = BUFFER_LEN;
 	}
 
+	/* JSON parser initialization */
+	init_JSON_config (&config);
+
+	nb_open_brackets              = -1;
+	config.depth                  = 20;
+	config.callback               = &json_char_callback;
+	config.allow_comments         = 0;
+	config.handle_floats_manually = 0;
+	config.callback_ctx           = &nb_open_brackets;
+
+	jc = new_JSON_parser (&config);
+
 	total = 0;
 	to_read = *len;
 	b = *buff;
@@ -626,12 +1098,14 @@ net_read_sock (int sock, char **buff, size_t *len)
 			lwc_writeLog (LOG_ERR,
 					_("Error while reading data: %s"),
 					strerror (errno));
+			delete_JSON_parser (jc);
 			return -1;
 		}
 
 		if (ret == 0) {
 			lwc_writeLog (LOG_NOTICE,
 			_("Timeout while waiting to read network data"));
+			delete_JSON_parser (jc);
 			return -1;
 		}
 
@@ -640,6 +1114,7 @@ net_read_sock (int sock, char **buff, size_t *len)
 		} while (nb_read < 0 && (errno == EINTR || errno == EAGAIN));
 
 		if (nb_read == 0) {
+			delete_JSON_parser (jc);
 			return total;
 		}
 
@@ -647,18 +1122,31 @@ net_read_sock (int sock, char **buff, size_t *len)
 			lwc_writeLog (	LOG_ERR,
 					_("Error while reading data: %s"),
 					strerror (errno));
+			delete_JSON_parser (jc);
 			return -1;
 		}
 
 		total += nb_read;
 
-		/* Find the EOF tag */
-		tmp = schedwi_strnstr (*buff, ENF_OF_REQUEST, total);
-		if (tmp != NULL) {
-			return (ssize_t)(tmp - *buff);
+		/*
+		 * Count the numbers of open/close brackets.  If this number
+		 * drops to zero it means that the full JSON request has
+		 * been received.
+		 */
+		for (i = 0; i < nb_read; i++) {
+			if (! JSON_parser_char (jc, b[i])) {
+				lwc_writeLog (	LOG_ERR,
+					_("Syntax error in received data"));
+				delete_JSON_parser (jc);
+				return -1;
+			}
+			if (nb_open_brackets == 0) {
+				delete_JSON_parser (jc);
+				return total;
+			}
 		}
-
-		if (nb_read < to_read) {
+		
+		if ((size_t)nb_read < to_read) {
 			to_read -= nb_read;
 			b += nb_read;
 		}
@@ -667,6 +1155,7 @@ net_read_sock (int sock, char **buff, size_t *len)
 				/* Too many data */
 				lwc_writeLog (LOG_NOTICE,
 		_("Failed to read the network request: buffer overflow"));
+				delete_JSON_parser (jc);
 				return -2;
 			}
 
@@ -674,6 +1163,7 @@ net_read_sock (int sock, char **buff, size_t *len)
 			if (tmp == NULL) {
 				lwc_writeLog (	LOG_CRIT,
 						_("Memory allocation error"));
+				delete_JSON_parser (jc);
 				return -1;
 			}
 			*buff = tmp;
@@ -683,6 +1173,8 @@ net_read_sock (int sock, char **buff, size_t *len)
 		}
 	} while (1);
 
+	delete_JSON_parser (jc);
+
 	return total;
 }
 
@@ -690,10 +1182,12 @@ net_read_sock (int sock, char **buff, size_t *len)
 /*
  * Read the reply
  *
- * The format of the reply must be as follow:
- *    The first character: '0' means success, an other character means failure
- *    The next characters: An optional explanation message (which is returned
- *                         in result_msg)
+ * The format of the JSON reply looks like:
+ *     {
+ *         "success" : false,
+ *          "data" : "Database unavailable"
+ *     }
+ *
  * No reply (nothing to read) means failure.
  *
  * Return:
@@ -708,10 +1202,10 @@ net_read_sock (int sock, char **buff, size_t *len)
 int
 net_read_result_sock (int sock, char **result_msg)
 {
-	char *buff, ret_code;
+	char *buff;
 	size_t len;
 	ssize_t nb_read;
-	int i;
+	int ret;
 
 	if (result_msg != NULL) {
 		*result_msg = NULL;
@@ -729,32 +1223,15 @@ net_read_result_sock (int sock, char **result_msg)
 	}
 
 	if (nb_read > 0) {
-		/* Return code sent by the remote process */
-		ret_code = buff[0];
-
-		if (result_msg != NULL) {
-			/* Shift the message 1 character to the left */
-			for (i = 1; i < nb_read; i++) {
-				buff[i - 1] = buff[i];
-			}
-			buff [nb_read - 1] = '\0';
-			*result_msg = buff;
-		}
-		else {
-			free (buff);
-		}
-
-		/* Return code '0' means success */
-		if (ret_code == '0') {
-			return 0;
-		}
+		ret = parse_json_result (buff, nb_read, result_msg, NULL);
 	}
 	else {
-		if (buff != NULL) {
-			free (buff);
-		}
+		ret = 1;
 	}
-	return 1;
+	if (buff != NULL) {
+		free (buff);
+	}
+	return ret;
 }
 
 
@@ -776,7 +1253,7 @@ net_write_sock (int sock, const char *buff, size_t len)
 
 	do {
 		nb_write = send (sock, buff, len, 0);
-		if (nb_write == len) {
+		if ((size_t)nb_write == len) {
 			return 0;
 		}
 
@@ -799,21 +1276,6 @@ net_write_sock (int sock, const char *buff, size_t len)
 }
 
 
-/*
- * Write the End of Request tag
- *
- * Return:
- *   0 --> No error
- *  -1 --> Write error (a message is displayed using lwc_writeLog())
- */
-int
-net_write_eof_sock (int sock)
-{
-	return net_write_sock (sock, ENF_OF_REQUEST,
-				(size_t)schedwi_strlen (ENF_OF_REQUEST));
-}
-
-
 /*
  * Close the socket
  */
