@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -24,10 +24,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -113,11 +109,38 @@ send_process_wakeup ()
 }
 
 
+/*
+ * Send the TERM signal to the process.
+ * Warning: the caller must also call the send_process_kill()
+ * function (see below)
+ */
+void
+send_process_term ()
+{
+	if (send_process_pid > 0) {
+		kill (send_process_pid, SIGTERM);
+	}
+}
+
+
+/*
+ * Send the TERM signal to the process
+ */
+void
+send_process_kill ()
+{
+	if (send_process_pid > 0) {
+		kill (send_process_pid, SIGKILL);
+		send_process_pid = 0;
+	}
+}
+
+
 /*
  * Signal handler for SIGTERM
  */
 static void
-sigterm_action (int sig)
+send_process_sigterm_action (int sig)
 {
 	conf_destroy_clnt ();
 	net_free ();
@@ -129,7 +152,7 @@ sigterm_action (int sig)
  * Signal handler for SIGHUP
  */
 static void
-sighup_action (int sig)
+send_process_sighup_action (int sig)
 {
 	return;
 }
@@ -150,7 +173,6 @@ send_process_start ()
 	const char *result_dir, *prefix, *suffix;
 	const char *log_file, *date_format;
 	long int facility;
-
 #if HAVE_NANOSLEEP
 	struct timespec req;
 
@@ -201,6 +223,19 @@ send_process_start ()
 			log_file,
 			date_format);
 
+	/* Install the signal handlers */
+	schedwi_memset (&sa, 0, sizeof (struct sigaction));
+	sa.sa_handler = send_process_sigterm_action;
+	sigemptyset (&(sa.sa_mask));
+	sa.sa_flags = 0;
+	sigaction (SIGTERM, &sa, 0);
+
+	schedwi_memset (&sa, 0, sizeof (struct sigaction));
+	sa.sa_handler = send_process_sighup_action;
+	sigemptyset (&(sa.sa_mask));
+	sa.sa_flags = 0;
+	sigaction (SIGHUP, &sa, 0);
+
 	/* Mask all the signals */
 	signal_unmask_all ();
 	sigfillset (&mask);
@@ -212,19 +247,6 @@ send_process_start ()
 	sigdelset (&mask, SIGHUP);
 	sigprocmask (SIG_SETMASK, &mask, 0);
 
-	/* Install the signal handlers */
-	schedwi_memset (&sa, 0, sizeof (struct sigaction));
-	sa.sa_handler = sigterm_action;
-	sigemptyset (&(sa.sa_mask));
-	sa.sa_flags = 0;
-	sigaction (SIGTERM, &sa, 0);
-
-	schedwi_memset (&sa, 0, sizeof (struct sigaction));
-	sa.sa_handler = sighup_action;
-	sigemptyset (&(sa.sa_mask));
-	sa.sa_flags = 0;
-	sigaction (SIGHUP, &sa, 0);
-
 	/* Main loop */
 	while (1) {
 
