@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -21,15 +21,15 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
 #endif
 
+#if HAVE_TIME_H
+#include <time.h>
+#endif
+
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -87,14 +87,19 @@ extern int errno;
 static int
 net_send_list (lwc_LL *jobs_to_insert)
 {
-	char *module;
-	ssize_t module_len;
-	char *txt;
+	char *txt, *server_reply;
 	size_t txt_len;
 	int ret;
 	file_result_t *ptr;
 	const char *hostname, *port_number;
 	net_id sock;
+#if HAVE_NANOSLEEP
+	struct timespec req;
+
+	req.tv_sec = 0;
+	req.tv_nsec = 500000000; /* 0.5 sec */
+#endif
+
 
 	if (jobs_to_insert == NULL || lwc_getNumNode (jobs_to_insert) == 0) {
 		return 0;
@@ -106,27 +111,6 @@ net_send_list (lwc_LL *jobs_to_insert)
 	assert (ret == 0);
 #endif
 
-	/* Construct the module name which will be send to the remote host */
-	module_len = build_module_name ("result", &module);
-	if (module_len < 0) {
-		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-		return -1;
-	}
-
-	/* Connect to the server */
-	if (net_client (&sock, port_number, hostname) != 0) {
-		free (module);
-		return -2;
-	}
-
-	/* Send the module name */
-	if (net_write (&sock, module, module_len) != 0) {
-		net_close (&sock);
-		free (module);
-		return -2;
-	}
-	free (module);
-
 	/* For each job, construct and send the result string */
 	lwc_rewindLL (jobs_to_insert);
 	txt = NULL;
@@ -135,7 +119,6 @@ net_send_list (lwc_LL *jobs_to_insert)
 
 		ret = result_to_str (ptr->r, &txt, &txt_len);
 		if (ret != 0) {
-			net_close (&sock);
 			if (txt != NULL) {
 				free (txt);
 			}
@@ -150,62 +133,71 @@ net_send_list (lwc_LL *jobs_to_insert)
 			return -1;
 		}
 
+		/* Connect to the server */
+		if (net_client (&sock, port_number, hostname) != 0) {
+			/*
+			 * Maybe the server if overloaded.
+			 * Throttle the client (sleep 0.5 s) and try again
+			 */
+#if HAVE_NANOSLEEP
+			nanosleep (&req, 0);
+#elif HAVE_SLEEP
+			sleep (1);
+#elif HAVE_USLEEP
+			usleep (500000);
+#endif
+			if (net_client (&sock, port_number, hostname) != 0) {
+				free (txt);
+				return -2;
+			}
+		}
+
+		/* Send the result */
 		if (net_write (&sock, txt, schedwi_strlen (txt)) != 0) {
 			net_close (&sock);
 			free (txt);
 			return -2;
 		}
-	}
 
-	if (txt != NULL) {
-		free (txt);
-	}
-
-	/* Send the End of Request tag to the server */
-	if (net_write_eof (&sock) != 0) {
+		/* Read the result code returned by the server */
+		server_reply = NULL;
+		ret = net_read_result (&sock, &server_reply);
 		net_close (&sock);
-		return -2;
-	}
-
-	/* Read the result code returned by the server */
-	txt = NULL;
-	ret = net_read_result (&sock, &txt);
-	net_close (&sock);
-	if (ret < 0) {
-		/* Network error */
-		if (txt != NULL) {
+		if (ret < 0) {
+			/* Network error */
 			free (txt);
+			if (server_reply != NULL) {
+				free (server_reply);
+			}
+			return -2;
 		}
-		return -2;
-	}
-	if (ret != 0) {
-		/* Server failure */
-		if (txt != NULL) {
-			lwc_writeLog (LOG_ERR, _("Server failure: %s"), txt);
+		if (ret != 0) {
+			/* Server failure */
 			free (txt);
+			if (server_reply != NULL) {
+				lwc_writeLog (LOG_ERR, _("Server failure: %s"),
+						server_reply);
+				free (server_reply);
+			}
+			else {
+				lwc_writeLog (LOG_ERR, _("Server failure"));
+			}
+			return -3;
 		}
-		else {
-			lwc_writeLog (LOG_ERR, _("Server failure"));
+		if (server_reply != NULL) {
+			free (server_reply);
 		}
-		return -3;
-	}
-	if (txt != NULL) {
-		free (txt);
-	}
 
-	/* Success from the server, remove the result files */
-	while ((ptr = (file_result_t *) lwc_delStartLL (jobs_to_insert))
-			!= NULL)
-	{
-		if (	   ptr->file_name != NULL
-			&& my_unlink (ptr->file_name) != 0)
-		{
+		/* Success from the server, remove the result file */
+		if (my_unlink (ptr->file_name) != 0) {
 			lwc_writeLog (	LOG_CRIT,
 				_("Cannot remove the result file %s: %s"),
 					ptr->file_name,
 					strerror (errno));
 		}
-		destroy_file_result (ptr);
+	}
+	if (txt != NULL) {
+		free (txt);
 	}
 
 	return 0;
@@ -263,7 +255,7 @@ send_result (const char *result_dir, const char *prefix, const char *suffix)
 	/* Read the directory */
 	while ((f = readdir (d)) != NULL) {
 		name_length = schedwi_strlen (f->d_name);
-		if (	   prefix_length + suffix_length >= name_length 
+		if (	   prefix_length + suffix_length >= name_length
 			|| schedwi_strncmp (	prefix, f->d_name,
 						prefix_length) != 0)
 		{
@@ -304,7 +296,7 @@ send_result (const char *result_dir, const char *prefix, const char *suffix)
 			destroy_file_result (ret_ptr);
 			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 			return -1;
-		} 
+		}
 	}
 	closedir (d);
 	ret = net_send_list (list);
